package com.ebaiyihui.family.doctor.server.service.impl;

import com.alibaba.fastjson.JSON;
import com.doctoruser.api.pojo.base.dto.doctor.QueryPersonnelInfoReq;
import com.doctoruser.api.pojo.base.vo.doctor.PersonnelInfo;
import com.ebaiyihui.family.doctor.common.dto.*;
import com.ebaiyihui.family.doctor.common.vo.*;
import com.ebaiyihui.family.doctor.server.common.constants.ProjProperties;
import com.ebaiyihui.family.doctor.server.entity.PatientEntity;
import com.ebaiyihui.family.doctor.server.entity.PatientSignEntity;
import com.ebaiyihui.family.doctor.server.feign.DoctorInfofeignClient;
import com.ebaiyihui.family.doctor.server.mapper.PatientMapper;
import com.ebaiyihui.family.doctor.server.mapper.PatientSignMapper;
import com.ebaiyihui.family.doctor.server.service.OrderManagementService;
import com.ebaiyihui.family.doctor.server.util.IDCardUtil;
import com.ebaiyihui.family.doctor.server.util.PageUtil;
import com.ebaiyihui.framework.response.BaseResponse;
import com.ebaiyihui.framework.utils.HttpKit;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

@Service
@Slf4j
public class OrderManagementServiceImpl implements OrderManagementService {

    @Autowired
    private PatientSignMapper patientSignMapper;

    @Autowired
    private PatientMapper patientMapper;

    @Autowired
    private ProjProperties projProperties;

    @Autowired
    private DoctorInfofeignClient doctorInfofeignClient;

    @Override
    public BaseResponse<PageUtil<OrderListResVo>> orderList(OrderListReqDTO param) {
        log.info("查询订单列表入参：{}", JSON.toJSONString(param));

        param.setPageNum((param.getPageNum() - 1) * param.getPageSize());
        List<OrderListResVo> records = patientSignMapper.orderList(param);
        log.info("records：{}", JSON.toJSONString(records));

        PageUtil<OrderListResVo> pageUtil = new PageUtil<>();
        pageUtil.setList(records);
        QueryOrderTotalVo totalVo = patientSignMapper.countSumOrder();
        pageUtil.setObject(totalVo);
        log.info("查询订单列表出参：{}", JSON.toJSONString(pageUtil));
        return BaseResponse.success(pageUtil);
    }

    @Override
    public BaseResponse<OrderDetailsResVo> getOrderDetails(OrderDetailResDTO param) {
        log.info("查询订单详情入参：{}", JSON.toJSONString(param));
        OrderDetailsResVo resVo = new OrderDetailsResVo();
        String orderId = param.getOrderId();
        PatientSignEntity patientSignEntity = patientSignMapper.selectById(orderId);
        if (Objects.isNull(patientSignEntity)) {
            return BaseResponse.error("订单不存在");
        }
        // 处方信息
        resVo.setPresListResVOS(getPresList(patientSignEntity.getAdmId()));
        // 订单信息
        resVo.setOrderInfo(buildOrderInfo(patientSignEntity));
        // 沟通记录 todo

        return BaseResponse.success(resVo);
    }


    private OrderInfo buildOrderInfo(PatientSignEntity patientSignEntity) {
        log.info("buildOrderInfo：{}", JSON.toJSONString(patientSignEntity));
        // 获取患者详情信息
        PatientEntity patient = patientMapper.selectById(patientSignEntity.getPatientId());
        log.info("patient：{}", JSON.toJSONString(patient));
        // 获取医生基本信息
        QueryPersonnelInfoReq queryPersonnelInfoReq = new QueryPersonnelInfoReq();
        queryPersonnelInfoReq.setDoctorId(String.valueOf(patientSignEntity.getDoctorId()));
        BaseResponse<PersonnelInfo> doctorInfo = doctorInfofeignClient.queryPersonnelInfo(queryPersonnelInfoReq);
        if (null == doctorInfo && !doctorInfo.isSuccess()) {
            log.error("IM INFORM ERROR : 获取医生信息失败 doctorId：{}", String.valueOf(patientSignEntity.getDoctorId()));
        }
        log.info("doctorInfo：{}", JSON.toJSONString(doctorInfo));

        OrderInfo orderInfo = new OrderInfo();
        orderInfo.setOrderNo(patientSignEntity.getId());
        orderInfo.setPatientPhone(patientSignEntity.getPatientPhone());
        orderInfo.setPatientSex(patient.getGender());
        orderInfo.setPatientName(patient.getPatientName());
        orderInfo.setPatientAge(IDCardUtil.getAge(patient.getCredNo()));
        orderInfo.setDoctorPhone(doctorInfo.getData().getPhoneNum());
        orderInfo.setDoctorPhone(patientSignEntity.getDoctorName());
        orderInfo.setDoctorTitle(doctorInfo.getData().getTitle());
        orderInfo.setHospital(patientSignEntity.getOrganName());
        orderInfo.setFirstDepartment(patientSignEntity.getDeptName());
        log.info("orderInfo：{}", JSON.toJSONString(orderInfo));
        return orderInfo;
    }

    private List<GetPresListResVO> getPresList(String admId) {
        GetPresListReqVO reqVo = new GetPresListReqVO();
        reqVo.setAdmId(admId);
        try {
            log.info("查询处方列表URL:{},入参{}, ", projProperties.getPresListByAdmId(), JSON.toJSONString(reqVo));
            String body = HttpKit.jsonPost(projProperties.getIcdSearchName(), JSON.toJSONString(reqVo));
            BaseResponse<GetPresListResVO> baseResponse = JSON.parseObject(body, BaseResponse.class);
            log.info("baseResponse:{} ",baseResponse);
            if (baseResponse.getData() == null) {
                return null;
            }
            List<GetPresListResVO> resVOList = (List<GetPresListResVO>) baseResponse.getData();
            log.info("查询处方列表返回值:{} ",resVOList);
            return resVOList;
        } catch (Exception e) {
            log.error("查询处方列表调用失败{}", e.getMessage());
        }
        return null;
    }
}
